import os
import re
from os import PathLike
from pathlib import Path
from typing import Union
from urllib.parse import urlparse

tenant_regex = re.compile(r"\[(.*)\]")


def get_tenant(auth_file: Union[PathLike, str]) -> str:
    with open(auth_file, "r") as f:
        for line in f:
            matches = re.match(tenant_regex, line)
            if matches:
                return matches.group(1)
    return ""


def get_ag_port(rpm_ini_file: Union[PathLike, str]) -> int:
    with open(rpm_ini_file, "r") as f:
        for line in f:
            if line.startswith("Server"):
                url = line.split("=")[1].strip()
                parsed = urlparse(url)
                return parsed.port
    return 9999


def find_authorization_file() -> Path:
    config_dir_base = os.path.expandvars("%PROGRAMDATA%") if os.name == "nt" else "/var/lib"
    return Path(config_dir_base) / "dynatrace" / "gateway" / "config" / "authorization.properties"


def find_rpm_ini_file() -> Path:
    config_dir_base = os.path.expandvars("%PROGRAMDATA%") if os.name == "nt" else "/var/lib"
    return Path(config_dir_base) / "dynatrace" / "remotepluginmodule" / "agent" / "conf" / "remotepluginuserconfig.ini"


def get_api_url():
    tenant = get_tenant(find_authorization_file())
    port = get_ag_port(find_rpm_ini_file())
    return f"https://localhost:{port}/e/{tenant}"
